
    {
      "requisiteStageRefIds":[""],
      "refId": "master",
      "type": "manualJudgment",
      "name": "Checkpoint {{ data.app.environment }}",
      "notifications": [],
      "judgmentInputs": [],
      "failPipeline": false,
      {% if data.app.approval_timeout %}
        "stageTimeoutMs": {{ data.app.approval_timeout }},
      {% endif %}
      "instructions": "I confirm that I'm ready for this application to be promoted to <strong>{{ data.app.environment }}</strong>.",
      "comments": ""
    }
