[
    {% if not data.app.approval_skip %}
        {% include "pipeline/stage-judgement-nonprod.json.j2" %},
    {% endif %}
    {% include "pipeline/stage-infrastructure-setup-lambda.json.j2" %},
    {% include "pipeline/stage-deploy-lambda.json.j2" %}
]
