{
    "Comment": "Automatic Update",
    "Changes": [
        {
            "Action": "UPSERT",
            "ResourceRecordSet": {
                "Name": "{{ dns_name }}",
                "Type": "CNAME",
                "TTL": {{ dns_ttl }},
                "ResourceRecords": [
                    {
                        "Value": "{{ dns_name_aws }}"
                    }
                ]
            }
        }
    ]
}
