[
    {% if data.app.previous_env %}
      {% if not data.app.approval_skip %}
        {% include "pipeline/stage-judgement-nonprod.json.j2" %},
      {% endif %}
    {% endif %}
    {% include "pipeline/stage-infrastructure-setup-s3.json.j2" %},
    {% if data.deploy_strategy == "canary" or data.deploy_strategy == "redblack" %}
      {% include "pipeline/stage-deploy-s3.json.j2" %},
      {% include "pipeline/stage-judgement-promote-s3-nonprod.json.j2" %},
      {% include "pipeline/stage-promote-s3.json.j2" %}
    {% else %}
      {% include "pipeline/stage-deploy-s3.json.j2" %}
    {% endif %}
    {% if 'completion_webhooks' in data and data.completion_webhooks %}
      ,{% include "pipeline/stage-completion-webhooks.json.j2" %}
    {% endif %}
]
