        {
            "Sid": "BrowseBucket",
            "Effect": "Allow",
            "Action": [
                "s3:ListBucket",
                "s3:GetBucketLocation",
                "s3:ListBucketMultipartUploads"
            ],
            "Resource": [
                {% for bucket in items -%}
                  "arn:aws:s3:::{{ bucket }}"
                  {%- if not loop.last -%}
                  ,
                  {%- endif -%}
                {% endfor -%}
            ]
        },
        {
            "Effect": "Allow",
            "Action": [
                "s3:DeleteObject",
                "s3:GetObject",
                "s3:PutObject",
                "s3:ListMultipartUploadParts",
                "s3:AbortMultipartUpload"
            ],
            "Resource": [
                {% for bucket in items -%}
                  "arn:aws:s3:::{{ bucket }}/*"
                  {%- if not loop.last -%}
                  ,
                  {%- endif -%}
                {% endfor -%}
            ]
        }
