        {
            "Sid": "AthenaLimitedAccess",
            "Action": [
                "athena:StartQueryExecution", 
                "athena:StopQueryExecution", 
                "athena:GetQueryExecution", 
                "athena:GetQueryResults", 
                "athena:RunQuery",
                "glue:Get*"
            ],
            "Effect": "Allow",
            "Resource": "*"
        }
